package to.newfs.webauth.colorselector;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import javax.swing.colorchooser.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.util.*;

class ColorValuePanel extends JPanel implements ActionListener {
  ValueField fg, bg, border;
  final JTextField tfld = new JTextField(); // for clipboard
  private PropertyChangeSupport psup;
  class ValueField extends JTextField {
    ValueField() {
      Dimension dim = this.getPreferredSize();
      dim.width = 32767;
      setMaximumSize(dim);
    }
  }
  ColorValuePanel() {
    super();
    setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    fg = new ValueField();
    fg.setAlignmentY(Component.CENTER_ALIGNMENT);
    fg.setAlignmentX(Component.LEFT_ALIGNMENT);
    bg = new ValueField();
    bg.setAlignmentY(Component.CENTER_ALIGNMENT);
    bg.setAlignmentX(Component.LEFT_ALIGNMENT);
    border = new ValueField();
    border.setAlignmentY(Component.CENTER_ALIGNMENT);
    border.setAlignmentX(Component.LEFT_ALIGNMENT);
    fg.setMaximumSize(fg.getPreferredSize());
    bg.setMaximumSize(fg.getPreferredSize());
    border.setMaximumSize(fg.getPreferredSize());
    FontMetrics fm = getFontMetrics(fg.getFont());
    Dimension pref = new Dimension(fm.charWidth('M') * 7,
				   fg.getPreferredSize().height);
    fg.setMaximumSize(pref);
    bg.setMaximumSize(pref);
    border.setMaximumSize(pref);
    fg.setMinimumSize(pref);
    bg.setMinimumSize(pref);
    border.setMinimumSize(pref);
    fg.setPreferredSize(pref);
    bg.setPreferredSize(pref);
    border.setPreferredSize(pref);
    // add(Box.createGlue());
    JButton b = new JButton("Foreground");
    b.setAlignmentY(Component.CENTER_ALIGNMENT);
    b.setAlignmentX(Component.LEFT_ALIGNMENT);
    b.setActionCommand(FieldSelector.fgcmd);
    b.addActionListener(this);
    JPanel pbox;
    pbox = new JPanel();
    pbox.setLayout(new BoxLayout(pbox, BoxLayout.Y_AXIS));
    pbox.add(b);
    pbox.add(fg);
    add(pbox);
    // add(Box.createGlue());
    b = new JButton("Background");
    b.setAlignmentY(Component.CENTER_ALIGNMENT);
    b.setAlignmentX(Component.LEFT_ALIGNMENT);
    b.setActionCommand(FieldSelector.bgcmd);
    b.addActionListener(this);
    pbox = new JPanel();
    pbox.setLayout(new BoxLayout(pbox, BoxLayout.Y_AXIS));
    pbox.add(b);
    pbox.add(bg);
    add(pbox);
    // add(Box.createGlue());
    b = new JButton("Border");
    b.setAlignmentY(Component.CENTER_ALIGNMENT);
    b.setAlignmentX(Component.LEFT_ALIGNMENT);
    b.setActionCommand(FieldSelector.bordercmd);
    b.addActionListener(this);
    pbox = new JPanel();
    pbox.setLayout(new BoxLayout(pbox, BoxLayout.Y_AXIS));
    pbox.add(b);
    pbox.add(border);
    add(pbox);
    // add(Box.createGlue());
    setBorder(new TitledBorder(LineBorder.createBlackLineBorder(),
			       "RGB Value Strings"));
    fg.addActionListener(this);
    bg.addActionListener(this);
    border.addActionListener(this);
    psup = new PropertyChangeSupport(this);
  }
  public void addPropertyChangeListener(PropertyChangeListener l) {
    psup.addPropertyChangeListener(l);
  }
  public void removePropertyChangeListener(PropertyChangeListener l) {
    psup.removePropertyChangeListener(l);
  }
  public void actionPerformed(ActionEvent ev) {
    Object src = ev.getSource();
    String cmd = ev.getActionCommand();
    if (src == fg) {
      psup.firePropertyChange(FieldSelector.fgcmd, null, cmd);
    } else if (src == bg) {
      psup.firePropertyChange(FieldSelector.bgcmd, null, cmd);
    } else if (src == border) {
      psup.firePropertyChange(FieldSelector.bordercmd, null, cmd);
    } else {
      String value;
      if (cmd == FieldSelector.fgcmd) {
	value = fg.getText();
      } else if (cmd == FieldSelector.bgcmd) {
	value = bg.getText();
      } else if (cmd == FieldSelector.bordercmd) {
	value = border.getText();
      } else {
	return;
      }
      tfld.setText(value);
      tfld.selectAll();
      tfld.copy();
    }
  }
  private void setLabelValue(ValueField label, Color col) {
    String hex = Integer.toHexString(col.getRGB()).substring(2);
    label.setText("#" + hex);
  }
  void setForegroundColor(Color col) {
    setLabelValue(fg, col);
  }
  void setBackgroundColor(Color col) {
    setLabelValue(bg, col);
  }
  void setBorderColor(Color col) {
    setLabelValue(border, col);
  }
}

class BorderWidthSelector extends JSlider implements ChangeListener {
  TitledBorder widthLabel;

  BorderWidthSelector(int min, int max, int value) {
    super(min, max, value);
    widthLabel = new TitledBorder(LineBorder.createBlackLineBorder(),
				  String.valueOf(value),
				  TitledBorder.RIGHT, TitledBorder.BOTTOM);
    setBorder(new TitledBorder(widthLabel, "Border width"));
    addChangeListener(this);
  }
  public void stateChanged(ChangeEvent e) {
    widthLabel.setTitle(String.valueOf(getValue()));
    repaint();
  }
}

class FieldSelector extends JPanel {
  static final String fgcmd = "foreground";
  static final String bgcmd = "background";
  static final String bordercmd = "border";
  static final int FOREGROUND = 0;
  static final int BACKGROUND = 1;
  static final int BORDER = 2;
  JRadioButton fg, bg, border;
  ButtonGroup bgroup;
  FieldSelector() {
    setLayout(new BoxLayout(this, BoxLayout.X_AXIS)); //FlowLayout(FlowLayout.LEFT));
    fg = new JRadioButton("Foreground");
    fg.setMnemonic('f');
    fg.setAlignmentY(Component.CENTER_ALIGNMENT);
    fg.setActionCommand(fgcmd);
    bg = new JRadioButton("Background");
    bg.setMnemonic('b');
    bg.setAlignmentY(Component.CENTER_ALIGNMENT);
    bg.setActionCommand(bgcmd);
    border = new JRadioButton("Border");
    border.setMnemonic('s');
    border.setAlignmentY(Component.CENTER_ALIGNMENT);
    border.setActionCommand(bordercmd);
    bgroup = new ButtonGroup();
    bgroup.add(fg);
    bgroup.add(bg);
    bgroup.add(border);
    add(fg);
    add(bg);
    add(border);
  }
  int getSelectedField() {
    ButtonModel selection = bgroup.getSelection();
    if (selection != null) {
      String cmd = bgroup.getSelection().getActionCommand();
      if (cmd == bgcmd) {
	return BACKGROUND;
      } else if (cmd == bordercmd) {
	return BORDER;
      } else {
	return FOREGROUND;
      }
    } else {
      return FOREGROUND; // Fail safe
    }
  }
  void addActionListener(ActionListener listener) {
    fg.addActionListener(listener);
    bg.addActionListener(listener);
    border.addActionListener(listener);
  }
  void setSelected(int sel) {
    switch (sel) {
     case FOREGROUND:
      bgroup.setSelected(fg.getModel(), true);
      break;
     case BACKGROUND:
      bgroup.setSelected(bg.getModel(), true);
      break;
     case BORDER:
      bgroup.setSelected(border.getModel(), true);
      break;
    }
  }
}

class DocumentPanel extends JPanel {
  JTextArea text;
  JPanel north, south, east, west;
  DocumentPanel() {
    text = new JTextArea();
    text.setLineWrap(true);
    setBackground(Color.white);
    setLayout(new BorderLayout());
    north = new JPanel(null);
    south = new JPanel(null);
    east = new JPanel(null);
    west = new JPanel(null);
    add(BorderLayout.CENTER, text);
    add(BorderLayout.NORTH, north);
    add(BorderLayout.SOUTH, south);
    add(BorderLayout.WEST, west);
    add(BorderLayout.EAST, east);
  }
  void setBorderSize(int top, int left, int bottom, int right) {
    Dimension topleft = new Dimension(left, top);
    Dimension bottomright = new Dimension(right, bottom);
    north.setPreferredSize(topleft/*new Dimension(0, top)*/);
    west.setPreferredSize(topleft/*new Dimension(left, 0)*/);
    south.setPreferredSize(bottomright/*new Dimension(0, bottom)*/);
    east.setPreferredSize(bottomright/*new Dimension(right, 0)*/);
    validate();
    invalidate();
  }
  Color getBorderColor() {
    return north.getBackground();
  }
  Color getBackgroundColor() {
    return text.getBackground();
  }
  Color getForegroundColor() {
    return text.getForeground();
  }
  void setBorderColor(Color col) {
    north.setBackground(col);
    south.setBackground(col);
    east.setBackground(col);
    west.setBackground(col);
  }
  void setBackgroundColor(Color col) {
    text.setBackground(col);
  }
  void setForegroundColor(Color col) {
    text.setForeground(col);
  }
  void setText(String str) {
    text.setText(str);
  }
  void setTextFont(Font f) {
    text.setFont(f);
  }
  String getText() {
    return text.getText();
  }
}

public class ColorSelector extends JFrame
  implements ChangeListener, ActionListener, PropertyChangeListener {
  JColorChooser chooser;
  JTextArea valueText;
  ColorSelectionModel cmodel;
  DocumentPanel text;
  FieldSelector fsel;
  BorderWidthSelector bsel;
  ColorValuePanel colp;
  static final String borderwstr = "borderWidth";
  static final String samplestr = "sampleText";
  static final String defaultSample = "̓TṽeLXgłB\nThis is sample Text.";
  private String propertyFile;
  static final String commentStr = "Properties for " + ColorSelector.class.getName();
  static final String version = "ColorSelector 1.0";
  static final int defaultBorderWidth = 30;

  public void propertyChange(PropertyChangeEvent ev) {
    String name = ev.getPropertyName();
    int hexval = Integer.parseInt(((String)ev.getNewValue()).substring(1), 16);
    int fval;
    if (name == FieldSelector.fgcmd) {
      fval = FieldSelector.FOREGROUND;
    } else if (name == FieldSelector.bgcmd) {
      fval = FieldSelector.BACKGROUND;
    } else if (name == FieldSelector.bordercmd) {
      fval = FieldSelector.BORDER;
    }  else {
      return;
    }
    setColor(fval, new Color(0xff000000|hexval));
  }
  public void actionPerformed(ActionEvent ev) {
    int selection = fsel.getSelectedField();
    switch (selection) {
     case FieldSelector.FOREGROUND:
      chooser.setColor(text.getForegroundColor());
      break;
     case FieldSelector.BACKGROUND:
      chooser.setColor(text.getBackgroundColor());
      break;
     case FieldSelector.BORDER:
      chooser.setColor(text.getBorderColor());
      break;
    }
  }
  void copyToClipboard() {
  }
  void setColor(int sel, Color col) {
    if (sel == fsel.getSelectedField()) {
      chooser.setColor(col);
    }
    switch (sel) {
     case FieldSelector.FOREGROUND:
      text.setForegroundColor(col);
      colp.setForegroundColor(col);
      break;
     case FieldSelector.BACKGROUND:
      text.setBackgroundColor(col);
      colp.setBackgroundColor(col);
      break;
     case FieldSelector.BORDER:
      text.setBorderColor(col);
      colp.setBorderColor(col);
      break;
    }
  }
  public void stateChanged(ChangeEvent ev) {
    Object src = ev.getSource();
    if (src == bsel) {
      int w = bsel.getValue();
      text.setBorderSize(w, w, w, w);
    } else if (src == cmodel) {
      setColor(fsel.getSelectedField(), chooser.getColor());
    }
  }
  class FixedWidthJTextArea extends JTextArea {
    FixedWidthJTextArea() {
      super();
    }
    public Dimension getPreferredSize() {
      FontMetrics metrics = getFontMetrics(getFont());
      Dimension dim = new Dimension(metrics.getMaxAdvance() * 20,
				    super.getPreferredSize().height);
      return dim;
    }
  }
  ColorSelector() {
    super(version);
    Container cpane = getContentPane();
    text = new DocumentPanel();
    Font font = new Font("Serif", Font.PLAIN, 16);
    text.setFont(new Font("Serif", Font.PLAIN, 16));
    text.setBorderSize(defaultBorderWidth, defaultBorderWidth, defaultBorderWidth, defaultBorderWidth);
    Box textBox = new Box(BoxLayout.Y_AXIS);
    chooser = new JColorChooser();
    chooser.addChooserPanel(new KeywordColorChooserPanel());
    cmodel = chooser.getSelectionModel();
    Box chooserBox = new Box(BoxLayout.Y_AXIS);
    fsel = new FieldSelector();
    fsel.setBorder(new TitledBorder(LineBorder.createBlackLineBorder(), "Select Field"));
    bsel = new BorderWidthSelector(0, 200, defaultBorderWidth);
    Box topBox = new Box(BoxLayout.X_AXIS);
    bsel.setAlignmentY(Component.TOP_ALIGNMENT);
    fsel.setAlignmentY(Component.TOP_ALIGNMENT);
    topBox.add(bsel);
    topBox.add(fsel);
    chooserBox.add(topBox);
    colp = new ColorValuePanel();
    colp.setAlignmentX(Component.LEFT_ALIGNMENT);
    text.setAlignmentX(Component.LEFT_ALIGNMENT);
    textBox.add(colp);
    textBox.add(text);
    chooserBox.add(chooser);
    chooser.setMaximumSize(chooser.getPreferredSize());
    chooserBox.add(Box.createGlue());
    cpane.setLayout(new BorderLayout());
    cpane.add(BorderLayout.WEST, chooserBox);
    cpane.add(BorderLayout.CENTER, textBox);
    cmodel.addChangeListener(this);
    bsel.addChangeListener(this);
    fsel.addActionListener(this);
    colp.addPropertyChangeListener(this);
    fsel.setSelected(FieldSelector.FOREGROUND);
    chooser.setColor(text.getForegroundColor());
    colp.setForegroundColor(text.getForegroundColor());
    colp.setBackgroundColor(text.getBackgroundColor());
    colp.setBorderColor(text.getBorderColor());
    addWindowListener(new WindowClosing());
  }
  class WindowClosing extends WindowAdapter {
    public void windowClosing(WindowEvent ev) {
      save();
      System.exit(0);
    }
  }
  public static void main(String args[]) {
    ColorSelector cs = new ColorSelector();
    if (args.length > 0) {
      cs.load(args[0]);
    } else {
      cs.load(null);
    }
    cs.pack();
    cs.setVisible(true);
  }
  void load(String fname) {
    propertyFile = fname;
    Properties prop = new Properties();
    if (fname != null) {
      try {
	FileInputStream in = new FileInputStream(fname);
	prop.load(in);
	in.close();
      } catch (IOException ex) {
      }
    }
    String value;
    int rgb;
    value = prop.getProperty(FieldSelector.fgcmd, "#000000");
    rgb = Integer.parseInt(value.substring(1), 16) | 0xff000000;
    setColor(FieldSelector.FOREGROUND, new Color(rgb));
    value = prop.getProperty(FieldSelector.bgcmd, "#FFFFFF");
    rgb = Integer.parseInt(value.substring(1), 16) | 0xff000000;
    setColor(FieldSelector.BACKGROUND, new Color(rgb));
    value = prop.getProperty(FieldSelector.bordercmd, "#555555");
    rgb = Integer.parseInt(value.substring(1), 16) | 0xff000000;
    setColor(FieldSelector.BORDER, new Color(rgb));
    value = prop.getProperty(borderwstr, String.valueOf(defaultBorderWidth));
    bsel.setValue(Integer.parseInt(value));
    value = prop.getProperty(samplestr, defaultSample);
    text.setText(value);
  }
  String hexStr6(int value) {
    String hex = Integer.toHexString(value);
    return hex.substring(2);
  }
  void save() {
    if (propertyFile != null) {
      Properties prop = new Properties();
      prop.put(FieldSelector.fgcmd,
	       "#" + hexStr6(text.getForegroundColor().getRGB()));
      prop.put(FieldSelector.bgcmd,
	       "#" + hexStr6(text.getBackgroundColor().getRGB()));
      prop.put(FieldSelector.bordercmd,
	       "#" + hexStr6(text.getBorderColor().getRGB()));
      prop.put(borderwstr, String.valueOf(bsel.getValue()));
      prop.put(samplestr, text.getText());
      try {
	FileOutputStream out = new FileOutputStream(propertyFile);
	prop.save(out, commentStr);
	out.close();
      } catch (IOException exp) {
	exp.printStackTrace(System.err);
      }
    }
  }
}
